/* This file produces Figure 1 of "Accounting for the Sources of Macroeconomic Tail Risks. It takes, as inputs, the output of read_data.do, mainly ioJ_data_linear.csv. 
The calculations for the skewness and kurtosis of gdp growth, produced in the first portion of the do file, are computed using data directly downloaded from FRED.
 */ 

set seed 1
ssc install freduse
local iterations=1000  /* number of bootstrapped samples */
set scheme s1mono

/* Annual =0 : Quarterly Data from 1972 to 2016
   Annual =1 : Annual Data from 1972 to 2016 
   Annual =2 : Annual Data from 1929 to 2016 */

forvalues annual=0(2)2 {

qui {

if `annual'==0 {
freduse GDPC1, clear
keep if year(daten)>=1972 & year(daten)<=2016
gen grow=log(GDPC1/GDPC1[_n-1])
sum grow, d

local mean_GDP=r(mean)
local std_GDP=r(sd)
local skew_GDP=r(skewness)
local kurt_GDP=r(kurtosis)

}

if `annual'==2 {
freduse GDPCA, clear
ren GDPCA GDPC1
keep if year(daten)>=1929 & year(daten)<=2016
gen grow=log(GDPC1/GDPC1[_n-1])
sum grow, d

local mean_GDP=r(mean)
local std_GDP=r(sd)
local skew_GDP=r(skewness)
local kurt_GDP=r(kurtosis)

}

forvalues iter=1/`iterations' {
   freduse GDPC1, clear
   if `annual'==0 {
    keep if year(daten)>=1972
   }
   if `annual'==1 {
    keep if substr(date,6,2)=="01" 
  } 
   if `annual'==2 {
   freduse GDPCA, clear
   ren GDPCA GDPC1
  }
   keep if year(daten)<=2016
   gen grow=log(GDPC1/GDPC1[_n-1])
   drop date 
   gen yr=year(daten)
   bsample  
   qui sum grow, d
   local mean`iter'=r(mean)
   local std`iter'=r(sd)

   local skew`iter'=r(skewness)
   local kurt`iter'=r(kurtosis)

}

expand 15

gen mean=.
gen std=.
gen skew=.
gen kurt=.

forvalues iter=1/`iterations' {
   replace mean=`mean`iter'' if _n==`iter'
   replace std=`std`iter'' if _n==`iter'

   replace skew=`skew`iter'' if _n==`iter'
   replace kurt=`kurt`iter'' if _n==`iter'
}

sum mean, d
local mean_GDP_5=r(p5)
local mean_GDP_95=r(p95)

sum std, d
local std_GDP_5=r(p5)
local std_GDP_95=r(p95)

sum skew, d
local skew_GDP_5=r(p5)
local skew_GDP_95=r(p95)

sum kurt, d
local kurt_GDP_5=r(p5)
local kurt_GDP_95=r(p95)

}

if `annual'==0 {
  di "Using Quarterly Data from 1972 to 2016"
} 
if `annual'==2 {
  di "Using Annual Data from 1929 to 2016"
} 

di "Skewness: `skew_GDP' , `skew_GDP_5' ,`skew_GDP_95'"
di "Kurtosis: `kurt_GDP' , `kurt_GDP_5' ,`kurt_GDP_95'"

}

forvalues iter=0/`iterations' {
insheet using ioJ_data_linear.csv, clear 
if `iter'>0 {
 bsample 
}
forvalues x=2/41 {
  qui sum v`x',d
  local skew_`x'_`iter'=r(skewness)
  local kurt_`x'_`iter'=r(kurtosis)
}
}
/* Need to expand the data set so that there are the necessary number of observations to store the bootstrapped skewness and kurtosis */
expand 15
forvalues x=2/41 {
   gen skew_`x'=.
   gen kurt_`x'=.
   forvalues iter=0/`iterations' {
       local y=`iter'+1
       replace skew_`x'=`skew_`x'_`iter'' if _n==`y'
       replace kurt_`x'=`kurt_`x'_`iter'' if _n==`y'
  } 
}

forvalues x=2/41 {
  sum skew_`x' if _n>=2 & _n<=1001, d
  replace skew_`x'=r(p5) if _n==1002
  replace skew_`x'=r(p95) if _n==1003
  sum kurt_`x' if _n>=2 & _n<=1001, d
  replace kurt_`x'=r(p5) if _n==1002
  replace kurt_`x'=r(p95) if _n==1003
}

keep skew* kurt*
keep if _n==1 | _n==1002 | _n==1003

forvalues x=2/41 {
   replace kurt_`x'=kurt_`x'-3
}

aorder
xpose, clear

gen x=mod(_n,40)
replace x=40.5 if x==0

graph twoway (pcspike  v2  x v3 x if _n<=40, horizontal color(black) )  (scatter x v1 if _n<=40  & v3*v2<0, msize(2) mcolor(black) msymbol(X))  (scatter x v1 if _n<=40 & v2*v3>0, msize(1) mcolor(black) msymbol(O)),  legend(off) xtitle("Kurtosis") yline(39.75, lpattern(dash)) xline(0, lpattern(dash)) ytitle("") ylab("") yline(24.5, lpattern(dash))

graph save "kurtosis_by_industry.gph", replace

graph twoway (pcspike  v2  x v3  x if _n>40, horizontal color(black) )  (scatter x v1 if _n>40  & v3*v2<0, msize(2) mcolor(black) msymbol(X))  (scatter x v1 if _n>40 & v2*v3>0, msize(1) mcolor(black) msymbol(O)),  legend(off) xtitle("Skewness")  ylab(1 "Mining" 2 "Utilities" 3 "Food Manuf." 4 "Beverage" 5 "Fabric" 6 "Textile" 7 "Apparel" 8 "Leather" 9 "Wood" 10 "Paper" 11 "Printing" 12 "Petroleum" 13 "Chemicals" 14 "Plastics & Rubber" 15 "Nonmetallic Mineral" 16 "Primary Metal" 17 "Fabricated Metal" 18 "Machinery" 19 "Computer & Electronics" 20 "Electrical Equipment" 21 "Transportation" 22 "Furniture" 23 "Misc. Manufacturing" 24 "Publishing" 25 "Agriculture" 26 "Construction" 27 "Wholesale" 28 "Retail" 29 "Transportation" 30 "Information" 31 "FIRE" 32 "Professional Services" 33 "Education" 34 "Health" 35 "Amusement" 36 "Accommodation" 37 "Food Service" 38 "Other Service" 39 "Government" 40.5 "GDP", angle(0) notick labsize(medsmall))  yline(39.75, lpattern(dash)) xline(0, lpattern(dash)) ytitle("") fxsize(75) yline(24.5, lpattern(dash))

graph save "skewness_by_industry.gph", replace

graph combine skewness_by_industry.gph kurtosis_by_industry.gph
graph export skew_kurt_figure_1.eps, replace
graph export skew_kurt_figure_1.tif, replace  width(2400) height(1800)

